#ifndef __BMP_H__
#define __BMP_H__

typedef struct {
    int width;
    int height;
    unsigned char *data;
} textureImage;

inline int _Error(const char *message, FILE *file) {
    printf(message);	
	fclose(file);
    return 0;
}

// simple BMP-loader. Refer to lesson06 for Linux at http://nehe.gamedev.net
int LoadBmp(char *filename, textureImage *texture) {
    FILE *file;
    unsigned short int bfType;
    long int bfOffBits;
    short int biPlanes;
    short int biBitCount;
    long int biSizeImage;
    int i;
    unsigned char temp;
    // make sure file exists.
    if (!(file = fopen(filename, "rb")))
        return _Error("File not found\n",file);
    // make sure file can be read
    if(!fread(&bfType, sizeof(short int), 1, file))
        return _Error("File could not be read\n",file);    
    // check if file is a bitmap
    if (bfType != 19778)
        return _Error("Not a bitmap\n",file);
    // get the file size
    // skip file size and reserved fields of bitmap file header
    fseek(file, 8, SEEK_CUR);
    // get the position of the actual bitmap data
    if (!fread(&bfOffBits, sizeof(long int), 1, file))        
        return _Error("File could not be read\n",file);        
    fseek(file, 4, SEEK_CUR);                       // skip size of bitmap info header    
    fread(&texture->width, sizeof(int), 1, file);   // get the width of the bitmap    
    fread(&texture->height, sizeof(int), 1, file);  // get the height of the bitmap  
    fread(&biPlanes, sizeof(short int), 1, file);   // get the number of planes
    if (biPlanes != 1)		
        return _Error("Number of bitplanes was not 1\n",file); 	
    // get the number of bits per pixel
    if (!fread(&biBitCount, sizeof(short int), 1, file))            
        return _Error("Error Reading file\n",file);    
    if (biBitCount != 24)		
        return _Error("File not 24Bpp\n",file);    	
    // calculate the size of the image in bytes
    biSizeImage = texture->width * texture->height * 3;    
    texture->data = (unsigned char*)malloc(biSizeImage);
    // seek to the actual data
    fseek(file, bfOffBits, SEEK_SET);
    if (!fread(texture->data, biSizeImage, 1, file))		
        return _Error("Error loading file\n",file);    
	fclose(file);
    // swap red and blue (bgr -> rgb)
    for (i = 0; i < biSizeImage; i += 3) {
        temp = texture->data[i];
        texture->data[i] = texture->data[i + 2];
        texture->data[i + 2] = temp;
    }
    return 1;	
}

#endif // defined __BMP_H__