/*************************************************************
 *                                                           *
 * file  : EXT_texture_env_combine.h                         *
 * author: Jens Schneider                                    *
 * date  : 01.Mar.2001                                       *
 * e-mail: jens@glHint.de                                    *
 *                                                           *
 * version 1.0 tested                                        *
 *                                                           *
 * ABOUT:                                                    *
 * ======                                                    *
 * Header-file to use the OpenGL-extension                   *
 * GL_EXT_texture_env_combine.                               *
 *                                                           *
 * See specification of the GL_EXT_texture_env_combine       * 
 * extension for more detail, available at www.opengl.org.   *
 *                                                           *
 *************************************************************/

#ifndef __EXT_TEXTURE_ENV_COMBINE_H_
#define __EXT_TEXTURE_ENV_COMBINE_H_

/*
 *  GLOBAL SWITCHES - enable/disable advanced features of this header
 *
 */
#define EXT_TEXTURE_ENV_COMBINE_INITIALIZE 1 // enable generic init-routines
//#define GL_GLEXT_PROTOTYPES 1

#ifdef __cplusplus
extern "C" {
#endif

#if defined(_WIN32) && !defined(APIENTRY) && !defined(__CYGWIN__)
#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>
#endif

#ifndef APIENTRY
#define APIENTRY
#endif


// Header file version number, required by OpenGL ABI for Linux
#define GL_GLEXT_VERSION 7


/* 
 * NEW TOKENS TO OPENGL 1.2.1 
 *
 */
#ifndef GL_EXT_texture_env_combine
#define GL_COMBINE_EXT                    0x8570
#define GL_COMBINE_RGB_EXT                0x8571
#define GL_COMBINE_ALPHA_EXT              0x8572
#define GL_RGB_SCALE_EXT                  0x8573
#define GL_ADD_SIGNED_EXT                 0x8574
#define GL_INTERPOLATE_EXT                0x8575
#define GL_CONSTANT_EXT                   0x8576
#define GL_PRIMARY_COLOR_EXT              0x8577
#define GL_PREVIOUS_EXT                   0x8578
#define GL_SOURCE0_RGB_EXT                0x8580
#define GL_SOURCE1_RGB_EXT                0x8581
#define GL_SOURCE2_RGB_EXT                0x8582
#define GL_SOURCE3_RGB_EXT                0x8583
#define GL_SOURCE4_RGB_EXT                0x8584
#define GL_SOURCE5_RGB_EXT                0x8585
#define GL_SOURCE6_RGB_EXT                0x8586
#define GL_SOURCE7_RGB_EXT                0x8587
#define GL_SOURCE0_ALPHA_EXT              0x8588
#define GL_SOURCE1_ALPHA_EXT              0x8589
#define GL_SOURCE2_ALPHA_EXT              0x858A
#define GL_SOURCE3_ALPHA_EXT              0x858B
#define GL_SOURCE4_ALPHA_EXT              0x858C
#define GL_SOURCE5_ALPHA_EXT              0x858D
#define GL_SOURCE6_ALPHA_EXT              0x858E
#define GL_SOURCE7_ALPHA_EXT              0x858F
#define GL_OPERAND0_RGB_EXT               0x8590
#define GL_OPERAND1_RGB_EXT               0x8591
#define GL_OPERAND2_RGB_EXT               0x8592
#define GL_OPERAND3_RGB_EXT               0x8593
#define GL_OPERAND4_RGB_EXT               0x8594
#define GL_OPERAND5_RGB_EXT               0x8595
#define GL_OPERAND6_RGB_EXT               0x8596
#define GL_OPERAND7_RGB_EXT               0x8597
#define GL_OPERAND0_ALPHA_EXT             0x8598
#define GL_OPERAND1_ALPHA_EXT             0x8599
#define GL_OPERAND2_ALPHA_EXT             0x859A
#define GL_OPERAND3_ALPHA_EXT             0x859B
#define GL_OPERAND4_ALPHA_EXT             0x859C
#define GL_OPERAND5_ALPHA_EXT             0x859D
#define GL_OPERAND6_ALPHA_EXT             0x859E
#define GL_OPERAND7_ALPHA_EXT             0x859F
#endif

#ifndef GL_EXT_texture_env_combine
#define GL_EXT_texture_env_combine 1

#ifdef GL_GLEXT_PROTOTYPES
#endif 

#endif // not GL_EXT_texture_env_combine

#ifdef EXT_TEXTURE_ENV_COMBINE_INITIALIZE
#include<string.h>		// string manipulation for runtime-check

#ifdef _WIN32
#endif // _WIN32

int CheckForEXTTextureEnvCombineSupport(void) {
	const char search[]="GL_EXT_texture_env_combine";
	char string[10000];
	int pos=0;
	int maxpos=strlen(search)-1;
	strcpy(string,(const char *)glGetString(GL_EXTENSIONS));
	int len=strlen(string);	
	for (int i=0; i<len; i++) {
		if ((i==0) || ((i>1) && string[i-1]==' ')) {
			pos=0;									
			while (string[i]!='\n') {				
				if (string[i]==search[pos]) pos++;	
				if ((pos>maxpos) && string[i+1]==' ') {
					printf(search);
					printf(" supported.\n");
					return 1;
				}
				i++;
			}			
		}
	}	
	fprintf(stderr,search);
	fprintf(stderr," not supported.\n");
	return 0;	
}

int GL_EXT_texture_env_combine_Init(void) {
	if (!CheckForEXTTextureEnvCombineSupport()) return 0;
	return 1;
}

#endif // EXT_TEXTURE_ENV_COMBINE_INITIALIZE

#ifdef __cplusplus
}

#endif

#endif // not __EXT_TEXTURE_ENV_COMBINE_H_
