/*************************************************************
 *                                                           *
 * file  : NV_evaluators.h                                   *
 * author: Jens Schneider                                    *
 * date  : 09.Apr.2001                                       *
 * e-mail: jens@glHint.de                                    *
 *                                                           *
 * version 1.0 tested                                        *
 *                                                           *
 * ABOUT:                                                    *
 * ======                                                    *
 * Header-file to use the OpenGL-extension                   *
 * GL_NV_evaluators.	                                     *
 *                                                           *
 * See specification at www.nvidia.com for more details.     *
 *                                                           *
 *************************************************************/

#ifndef __NV_EVALUATORS_H_
#define __NV_EVALUATORS_H_

/*
 *  GLOBAL SWITCHES - enable/disable advanced features of this header
 *
 */
#define NV_EVALUATORS_INITIALIZE 1 // enable generic init-routines
//#define GL_GLEXT_PROTOTYPES 1

#ifdef __cplusplus
extern "C" {
#endif

#if defined(_WIN32) && !defined(APIENTRY) && !defined(__CYGWIN__)
#define WIN32_LEAN_AND_MEAN 1
#include <windows.h>
#endif

#ifndef APIENTRY
#define APIENTRY
#endif


// Header file version number, required by OpenGL ABI for Linux
#define GL_GLEXT_VERSION 7


/* 
 * NEW TOKENS TO OPENGL 1.2.1 
 *
 */
#ifndef GL_NV_evaluators
#define GL_EVAL_2D_NV                        0x86C0
#define GL_EVAL_TRIANGULAR_2D_NV             0x86C1
#define GL_MAP_TESSELLATION_NV               0x86C2
#define GL_MAP_ATTRIB_U_ORDER_NV             0x86C3
#define GL_MAP_ATTRIB_V_ORDER_NV             0x86C4
#define GL_EVAL_FRACTIONAL_TESSELLATION_NV   0x86C5
#define GL_EVAL_VERTEX_ATTRIB0_NV            0x86C6
#define GL_EVAL_VERTEX_ATTRIB1_NV            0x86C7
#define GL_EVAL_VERTEX_ATTRIB2_NV            0x86C8
#define GL_EVAL_VERTEX_ATTRIB3_NV            0x86C9
#define GL_EVAL_VERTEX_ATTRIB4_NV            0x86CA
#define GL_EVAL_VERTEX_ATTRIB5_NV            0x86CB
#define GL_EVAL_VERTEX_ATTRIB6_NV            0x86CC
#define GL_EVAL_VERTEX_ATTRIB7_NV            0x86CD
#define GL_EVAL_VERTEX_ATTRIB8_NV            0x86CE
#define GL_EVAL_VERTEX_ATTRIB9_NV            0x86CF
#define GL_EVAL_VERTEX_ATTRIB10_NV           0x86D0
#define GL_EVAL_VERTEX_ATTRIB11_NV           0x86D1
#define GL_EVAL_VERTEX_ATTRIB12_NV           0x86D2
#define GL_EVAL_VERTEX_ATTRIB13_NV           0x86D3
#define GL_EVAL_VERTEX_ATTRIB14_NV           0x86D4
#define GL_EVAL_VERTEX_ATTRIB15_NV           0x86D5
#define GL_MAX_MAP_TESSELLATION_NV           0x86D6
#define GL_MAX_RATIONAL_EVAL_ORDER_NV        0x86D7
#endif


#ifndef GL_NV_evaluators
#define GL_NV_evaluators 1

#ifdef GL_GLEXT_PROTOTYPES
extern void APIENTRY glMapControlPointsNV(GLenum, GLuint, GLenum, GLsizei, GLsizei, GLint, GLint, GLboolean, const GLvoid *);
extern void APIENTRY glMapParameterivNV(GLenum, GLenum, const GLint *);
extern void APIENTRY glMapParameterfvNV(GLenum, GLenum, const GLfloat *);
extern void APIENTRY glGetMapControlPointsNV(GLenum, GLuint, GLenum, GLsizei, GLsizei, GLboolean, GLvoid *);
extern void APIENTRY glGetMapParameterivNV(GLenum, GLenum, GLint *);
extern void APIENTRY glGetMapParameterfvNV(GLenum, GLenum, GLfloat *);
extern void APIENTRY glGetMapAttribParameterivNV(GLenum, GLuint, GLenum, GLint *);
extern void APIENTRY glGetMapAttribParameterfvNV(GLenum, GLuint, GLenum, GLfloat *);
extern void APIENTRY glEvalMapsNV(GLenum, GLenum)
#endif // GL_GLEXT_PROTOTYPES 

typedef void (APIENTRY * PFNGLMAPCONTROLPOINTSNV) (GLenum target, GLuint index, GLenum type, GLsizei ustride, GLsizei vstride, GLint uorder, GLint vorder, GLboolean packed, const GLvoid *points);
typedef void (APIENTRY * PFNGLMAPPARAMETERIVNV) (GLenum target, GLenum pname, const GLint *params);
typedef void (APIENTRY * PFNGLMAPPARAMETERFVNV) (GLenum target, GLenum pname, const GLfloat *params);
typedef void (APIENTRY * PFNGLGETMAPCONTROLPOINTSNV) (GLenum target, GLuint index, GLenum type, GLsizei ustride, GLsizei vstride, GLboolean packed, GLvoid *points);
typedef void (APIENTRY * PFNGLGETMAPPARAMETERIVNV) (GLenum target, GLenum pname, GLint *params);
typedef void (APIENTRY * PFNGLGETMAPPARAMETERFVNV) (GLenum target, GLenum pname, GLfloat *params);
typedef void (APIENTRY * PFNGLGETMAPATTRIBPARAMETERIVNV) (GLenum target, GLuint index, GLenum pname, GLint *params);
typedef void (APIENTRY * PFNGLGETMAPATTRIBPARAMETERFVNV) (GLenum target, GLuint index, GLenum pname, GLfloat *params);
typedef void (APIENTRY * PFNGLEVALMAPSNV) (GLenum, GLenum);
#endif // not GL_NV_evaluators

#ifdef NV_EVALUATORS_INITIALIZE
#include<string.h>		// string manipulation for runtime-check

#ifdef _WIN32
PFNGLMAPCONTROLPOINTSNV glMapControlPointsNV = NULL;
PFNGLMAPPARAMETERIVNV glMapParameterivNV = NULL;
PFNGLMAPPARAMETERFVNV  glMapParameterfvNV = NULL;
PFNGLGETMAPCONTROLPOINTSNV glGetMapControlPointsNV = NULL;
PFNGLGETMAPPARAMETERIVNV glGetMapParameterivNV = NULL;
PFNGLGETMAPPARAMETERFVNV glGetMapParameterfvNV = NULL;
PFNGLGETMAPATTRIBPARAMETERIVNV glGetMapAttribParameterivNV = NULL;
PFNGLGETMAPATTRIBPARAMETERFVNV glGetMapAttribParameterfvNV = NULL;
PFNGLEVALMAPSNV glEvalMapsNV = NULL;
#endif // _WIN32

int CheckForNVEvaluatorsSupport(void) {
	const char search[]="GL_NV_evaluators";
	char string[10000];
	int pos=0;
	int maxpos=strlen(search)-1;
	strcpy(string,(const char *)glGetString(GL_EXTENSIONS));
	int len=strlen(string);	
	for (int i=0; i<len; i++) {
		if ((i==0) || ((i>1) && string[i-1]==' ')) {
			pos=0;									
			while (string[i]!='\n') {				
				if (string[i]==search[pos]) pos++;	
				if ((pos>maxpos) && string[i+1]==' ') {
					printf(search);
					printf(" supported.\n");
					return 1;
				}
				i++;
			}			
		}
	}	
	fprintf(stderr,search);
	fprintf(stderr," not supported.\n");
	return 0;	
}
	
int GL_NV_evaluators_Init(void) {
	if (!CheckForNVEvaluatorsSupport()) return 0;

#ifdef _WIN32
    glMapControlPointsNV = (PFNGLMAPCONTROLPOINTSNV) wglGetProcAddress("glMapControlPointsNV");
   	        if (glMapControlPointsNV==NULL) {fprintf(stderr,"glMapControlPointsNV not found.\n"); return 0; }
    glMapParameterivNV = (PFNGLMAPPARAMETERIVNV) wglGetProcAddress("glMapParameterivNV");
   	        if (glMapParameterivNV==NULL) {fprintf(stderr,"glMapParameterivNV not found.\n"); return 0; }
    glMapParameterfvNV = (PFNGLMAPPARAMETERFVNV) wglGetProcAddress("glMapParameterfvNV");
   	        if (glMapParameterfvNV==NULL) {fprintf(stderr,"glMapParameterfvNV not found.\n"); return 0; }
    glGetMapControlPointsNV = (PFNGLGETMAPCONTROLPOINTSNV) wglGetProcAddress("glGetMapControlPointsNV");
            if (glGetMapControlPointsNV==NULL) {fprintf(stderr,"glGetMapControlPointsNV not found.\n"); return 0;}
 	glGetMapParameterivNV = (PFNGLGETMAPPARAMETERIVNV) wglGetProcAddress("glGetMapParameterivNV");
	        if (glGetMapParameterivNV==NULL) {fprintf(stderr,"glGetMapParameterivNV not found.\n"); return 0;}
 	glGetMapParameterfvNV = (PFNGLGETMAPPARAMETERFVNV) wglGetProcAddress("glGetMapParameterfvNV");
	        if (glGetMapParameterfvNV==NULL) {fprintf(stderr,"glGetMapParameterfvNV not found.\n"); return 0;}
 	glGetMapAttribParameterivNV = (PFNGLGETMAPATTRIBPARAMETERIVNV) wglGetProcAddress("glGetMapAttribParameterivNV");
	        if (glGetMapAttribParameterivNV==NULL) {fprintf(stderr,"glGetMapAttribParameterivNV not found.\n"); return 0;}
 	glGetMapAttribParameterfvNV = (PFNGLGETMAPATTRIBPARAMETERFVNV) wglGetProcAddress("glGetMapAttribParameterfvNV");
	        if (glGetMapAttribParameterfvNV==NULL) {fprintf(stderr,"glGetMapAttribParameterfvNV not found.\n"); return 0;}
    glEvalMapsNV = (PFNGLEVALMAPSNV) wglGetProcAddress("glEvalMapsNV");
            if (glEvalMapsNV==NULL) {fprintf(stderr,"glEvalMapsNV not found.\n"); return 0;}
#endif // _WIN32
	return 1;
}

#endif // NV_EVALUATORS_INITIALIZE

#ifdef __cplusplus
}

#endif

#endif // not __NV_EVALUATORS_H_
